/* Shadow Game written by Mike Buerli
 * 471 - Final Project Fall 2010 */
#include "obj.h"
#define RSPEED .2
#define GRAVACC .03
#define GRAVTERM 0.9
#define MAXJUMP .2
#define MAXSPEED .3
#define TIMERINT 50
#define LSPEED 2
#define IDLEACC 0.00005
#define IDLESPEED 0.005
#define RAINDENS 10
//make moving platforms .15
typedef struct materialStruct {
  GLfloat ambient[4];
  GLfloat diffuse[4];
  GLfloat specular[4];
  GLfloat shininess[1];
} materialStruct;

materialStruct mNone = {
  {0.0, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};

materialStruct mBlack = {
  {0.2, 0.2, 0.2, 1.0},
  {0.2, 0.2, 0.2, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};
materialStruct mWhite = {
  {1, 1, 1, 1.0},
  {1, 1, 1, 1.0},
  {1, 1, 1, 1.0},
  {10}
};
materialStruct mGold = {
  {0.24725, 0.1995, 0.0745, 1.0},
  {0.75164, 0.60648, 0.22648, 1.0},
  {0.628281, 0.555802, 0.366065, 1.0},
  {60}
};
materialStruct mGrey = {
  {0.4, 0.4, 0.4, 1.0},
  {0.6, 0.6, 0.6, 1.0},
  {0.6, 0.6, 0.6, 1.0},
  {0.0}
};
materialStruct mRed = {
  {0.3, 0.0, 0.0, 1.0},
  {0.9, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};
materialStruct mGreen = {
  {0.0, 0.0, 0.0, 1.0},
  {0.1, 0.35, 0.1, 1.0},
  {0.45, 0.55, 0.45, 1.0},
  {32}
};
materialStruct mBrown = {
  {0.0, 0.0, 0.0, 1.0}, 	
  {0.714, 0.4284, 0.18144, 1.0},
  {0.393548, 0.271906, 0.166721, 1.0},
  {25}
};
materialStruct mChrome = {
  {0.25, 0.25, 0.25, 1.0}, 	
  {0.4, 0.4, 0.4, 1.0},
  {0.774597, 0.774597, 0.774597, 1.0},
  {77}
};

// lighting---------------
//globals for lighting - use a white light and apply materials
//light position
GLfloat light_pos[4] = {0.0, 2.0, 0.0, 1.0};
//light color (ambiant, diffuse and specular)
GLfloat light_amb[4] = {0, 0.9, 0.9, 1.0};
GLfloat light_diff[4] = {0.9, 0.9, 0.9, 1.0};
GLfloat light_spec[4] = {0.9, 0.9, 0.9, 1.0};

GLUquadricObj* lSphere;

// end lighting --------
std::vector<obj> objs;
std::vector<rainType> rain;
std::vector<pnt3d> path;
//other globals
int GW;
int GH;

bool stencil;
//int tp;


bool trann;
vec3d cam;
float theta,phi;
float lastx, lasty;
int deg;
float grav;
GLUquadricObj *quad;

int dirx;
int dirz;

bool mypause;
bool idleL;
int ltog;
int lindex;
float ldx;
float ldy;
float ldz;
float offset;
float ospeed;

bool wireframe = false;
bool lightdif = true;
void reshape(int w, int h);
void update(int n);
bool rainOn = false;
//lighting 

float distance(float x,float y,float z,float x2,float y2,float z2){
   return sqrt((x - x2)*(x - x2) + (y - y2)*(y - y2) + (z - z2)*(z - z2));
}

void materials(materialStruct materials) {
  glMaterialfv(GL_FRONT, GL_AMBIENT, materials.ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, materials.diffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR, materials.specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, materials.shininess);
}

void init_lighting() {
  //turn on light0
  glEnable(GL_LIGHT0);
  //set up the diffuse, ambient and specular components for the light
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);
  //glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.0f);
  //glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.2f);
  glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.01f);
  //specify our lighting model as 1 normal per face
  glShadeModel(GL_SMOOTH);
  //glEnable(GL_NORMALIZE);
}

void pos_light() {
  //set the light's position
  glMatrixMode(GL_MODELVIEW);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);
  glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
    glPushMatrix();
     glTranslatef(light_pos[0],light_pos[1],light_pos[2]);
      materials(mWhite);
      glBindTexture(GL_TEXTURE_2D, 2);
      gluSphere(lSphere, .1, 12, 12);
  glPopMatrix();
}

void movingObj(obj* o, float nx, float nz){
   o->mx = nx;
   o->mz = nz;
   o->moving = true;
}

void makeYO(float ox,float oy,float oz,float ow,float oh,float ol){
   objs.push_back(newCube(ox,oy,oz,          1,oh,ol));
   objs.push_back(newCube(ox+ow-1,oy,oz,     1,oh,ol));
   objs.push_back(newCube(ox+1,oy,oz,        ow-2,oh,1));
   objs.push_back(newCube(ox+1,oy, oz+ol-1,  ow-2,oh,1));
}

void makeZO(float ox,float oy,float oz,float ow,float oh,float ol){
   objs.push_back(newCube(ox,       oy,      oz,      1,    oh,   ol));
   objs.push_back(newCube(ox+ow-1,  oy,      oz,      1,    oh,   ol));
   objs.push_back(newCube(ox+1,     oy,      oz,      ow-2, 1,   ol));
   objs.push_back(newCube(ox+1,     oy+oh-1, oz,      ow-2, 1,   ol));
}

void makeXO(float ox,float oy,float oz,float ow,float oh,float ol){
   objs.push_back(newCube(ox,  oy,      oz,      ow,    oh,   1));
   objs.push_back(newCube(ox,  oy,      oz+ol-1, ow,    oh,   1));
   objs.push_back(newCube(ox,  oy,      oz+1,    ow, 1,   ol-2));
   objs.push_back(newCube(ox,  oy+oh-1, oz+1,    ow, 1,   ol-2));
}

void constructScene(){
   objs.push_back(newCube(-3,-1,3,6,8,1));
   makeZO(-3,-1,-7,6,8,10);
   makeZO(-4,-2,-17,8,9,10);
   objs.push_back(newCube(-2,-1,-9,1,7,1));
   objs.push_back(newCube(1,-1,-9,1,7,1));
   objs.push_back(newCube(-2,-1,-11,1,7,1));
   objs.push_back(newCube(1,-1,-11,1,7,1));
   objs.push_back(newCube(-2,-1,-13,1,7,1));
   objs.push_back(newCube(1,-1,-13,1,7,1));

   objs.push_back(newCube(-4,-2,-20,10,1,3));
   objs.push_back(newCube(-4,-2,-24,10,9,1));

   objs.push_back(newCube(-4,0,-20.5,10,.2,.2));
   objs.push_back(newCube(-4,1,-20.5,10,.2,.2));
   objs.push_back(newCube(-4,2,-20.5,10,.2,.2));
   objs.push_back(newSlope(4,-1,-17,6,3,-10,0,1));

   objs.push_back(newCube(6,-2,-20,.5,5,18));
   objs.push_back(newCube(4,2,-10,2,1,3));
   objs.push_back(newSlope(3,1,-7,6,3,-2,0,0));

   objs.push_back(newCube(3,0.5,-2,3,0.5,6));
   objs.push_back(newCube(3,0,-2,4,0.5,7));
   objs.push_back(newCube(3,-0.5,-2,5,0.5,8));
   objs.push_back(newCube(3,-1,-2,6,0.5,9));
   objs.push_back(newCube(3,-1.5,-2,7,0.5,10));
   objs.push_back(newCube(3,-2,-2,8,0.5,11));

//up spiral
   objs.push_back(newCube(17,2.5,-15,3,.5,3));
   movingObj(&objs[objs.size()-1],-10,0);
   objs.push_back(newCube(17,2.5,-10,3,.5,3));
   movingObj(&objs[objs.size()-1],10,0);
   objs.push_back(newCube(17,2.5,-5,3,.5,3));
   movingObj(&objs[objs.size()-1],-10,0);

   objs.push_back(newCube(30,2.5,-17,5,.5,15));
   objs.push_back(newCube(33,-15,-17,1,17.5,15));//platform after sliders
   
   objs.push_back(newSlope(31,3,-2,34,7,5,0,1));//up spiral
   objs.push_back(newCube(30,3,-2,1,4,7));//outer
   objs.push_back(newCube(34,3,-2,1,4,7));//innera
   objs.push_back(newCube(30,6,5,4,1,4));//plat
   
   objs.push_back(newSlope(34,7,5,41,11,8,1,1));
   objs.push_back(newCube(34,7,4,7,4,1));//inner
   objs.push_back(newCube(34,7,8,7,4,1));//outer
   objs.push_back(newCube(41,10,5,4,1,4));//plat
   
   objs.push_back(newSlope(41,11,-2,44,15,5,0,0));
   objs.push_back(newCube(40,11,-2,1,4,7));//inner
   objs.push_back(newCube(44,11,-2,1,4,7));//outer
   objs.push_back(newCube(41,14,-6,4,1,4));//plat

   objs.push_back(newSlope(34,15,-5,41,19,-2,1,0));
   objs.push_back(newCube(34,15,-6,7,4,1));//inner
   objs.push_back(newCube(34,15,-2,7,4,1));//outer
   objs.push_back(newCube(19,18.5,-6,15,.5,5));//plat

   makeXO(28,16,-7,1,10,10);
   makeXO(26,16,-8,1,10,10);
   makeXO(24,16,-9,1,10,10);
   makeXO(22,16,-10,1,10,10);

   objs.push_back(newCube(19,18.5,-20,3,.5,3));
   movingObj(&objs[objs.size()-1],0,10);
   objs.push_back(newCube(14,18.5,-22,3,.5,3));
   movingObj(&objs[objs.size()-1],0,-5);
   objs.push_back(newCube(9,18.5,-17,3,.5,3));
   movingObj(&objs[objs.size()-1],0,15);
   objs.push_back(newCube(4,18.5,-12,3,.5,3));
   movingObj(&objs[objs.size()-1],0,-7);
   
   objs.push_back(newCube(0,18.5,-7,3,.5,5));
   objs.push_back(newCube(1,7,-5,.5,11.5,2));
   objs.push_back(newSlope(0,19,-21,3,30,-7,0,0));
   makeYO(-3,24,-16,10,1,10);
   makeYO(-3,27,-20,10,1,10);
   objs.push_back(newCube(0,29,-30,3,1,9));
   objs.push_back(newCube(13,29.5,-30,3,.5,3));
   movingObj(&objs[objs.size()-1],10,0);
   objs.push_back(newCube(13,29.5,-27,3,.5,3));
   movingObj(&objs[objs.size()-1],-5,0);
   objs.push_back(newCube(20,29.5,-24,3,.5,3));
   movingObj(&objs[objs.size()-1],-10,0);
   objs.push_back(newCube(15,30,-30,.5,3,6));
   objs.push_back(newCube(33,29,-30,3,1,9));

   makeZO(29,30,-20,10,10,1);
   makeZO(30,31,-17,8,8,1);
   makeZO(31,32,-14,6,6,1);

   makeYO(15,-1,-3,10,1,10);
   makeYO(15,4,-3,10,1,10);

   makeZO(15,-1,10,10,10,1);
   makeZO(15,-1,15,10,10,1);

   objs.push_back(newSlope(35,0,-17,40,3,-2,1,0));
   objs.push_back(newSlope(40,-2,-17,46,0,-2,1,0));
   objs.push_back(newCube(46,-3,-17,8,1,15));
   objs.push_back(newSlope(54,-2,-17,60,0,-2,1,1));
   objs.push_back(newSlope(60,0,-17,65,3,-2,1,1));
}

void buildPath(){
   path.push_back(pnt3d(0,2,0));
   path.push_back(pnt3d(0,0,-18));
   path.push_back(pnt3d(5,0,-18));
   path.push_back(pnt3d(5,4.5,-8));
   path.push_back(pnt3d(33,4.5,-8));
   path.push_back(pnt3d(32.5,8,6.5));
   path.push_back(pnt3d(42.5,12,6.5));
   path.push_back(pnt3d(42.5,16,-3.5));
   path.push_back(pnt3d(32,20,-3.5));
   path.push_back(pnt3d(1,20,-3.5));
   path.push_back(pnt3d(1,31,-22));
   path.push_back(pnt3d(34,31,-22));
   path.push_back(pnt3d(34,34,-13.5));
}

void drawRain(rainType r){
   if (r.hit == 0){
      glBegin(GL_LINES);
         glNormal3f(0,1,0);
         glVertex3f(r.x, r.y, r.z);
	      glVertex3f(r.x, r.y + r.l, r.z);
      glEnd();
   } else {
      float cx;
      float cz;
      glBegin(GL_LINES);
         glNormal3f(0,1,0);
         for (int i = 0; i < 360; i+=5){
            cx = .02*r.hit*cos(DEG2RAD*i);
            cz = .02*r.hit*sin(DEG2RAD*i);
            glVertex3f(r.x + cx, r.y, r.z + cz);
         }
      glEnd();
   }
}

void drawFloor(){
  materials(mGrey);
  glBegin(GL_POLYGON);
	glVertex3f(50, -0.05, 50);
	glVertex3f(50, -0.05, -50);
	glVertex3f(-50, -0.05, -50);
	glVertex3f(-50, -0.05, 50);
  glEnd();
  materials(mBlack);
  glBegin(GL_LINES);
	for (int i = -50; i < 50; i++){
		glVertex3f(i, 0, 50);
		glVertex3f(i, 0, -50);
		glVertex3f(i + .5, 0, -50);
		glVertex3f(i + .5, 0, 50);
	}
	for (int i = -50; i < 50; i++){
		glVertex3f(50, 0, i);
		glVertex3f(-50, 0, i);
		glVertex3f(-50, 0, i+.5);
		glVertex3f(50, 0, i+.5);
	}
  glEnd();
}

void display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
  //glDisable(GL_STENCIL_TEST);
  glMatrixMode(GL_MODELVIEW);
    
  glPushMatrix();
  //set up the camera
  

  gluLookAt(cam.px, cam.py, cam.pz,//cam.x+cam.px, cam.y+cam.py, cam.z+cam.pz, //eye position
    cam.px-cam.x, cam.py-cam.y, cam.pz-cam.z,//cam.px, cam.py, cam.pz, //reference point
    0.0, 1.0, 0.0);  //up vector
  //glDepthMask(GL_FALSE);
  pos_light();
  glLoadName(-2);
  //glDepthMask(GL_FALSE);
  //drawFloor();
  //glDepthMask(GL_TRUE);
  //draw objects
  materials(mGrey);
  for (unsigned int i = 0; i < objs.size(); i++){
     objs[i].drawObj(i);
  }
  materials(mWhite);
  for (unsigned int i = 0; i < rain.size(); i++){
     drawRain(rain[i]);
  }

  glLoadName(-1);
  //draw shadows
  materials(mNone);
  for (unsigned int i = 0; i < objs.size(); i++){
     if (distance(objs[i].x,objs[i].y,objs[i].z,cam.px,cam.py,cam.pz)<M_INFINITY-10){
         objs[i].drawShadow(light_pos[0],light_pos[1],light_pos[2]);
     }
  }

  glPopMatrix();
    
  glutSwapBuffers();
    
}

int selector(float x,float y){
 	GLuint buff[64] = {0};
 	GLint hits, view[4];
 	glSelectBuffer(64, buff);//set buffer
 	glRenderMode(GL_SELECT);//enter select mode
 	
   glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
   
   glGetIntegerv(GL_VIEWPORT,view);
	gluPickMatrix(x,view[3]-y,
			2,2,view);
	gluPerspective(60, (float)GW/GH, 0.1, 40);

   glInitNames();//start names
 	glPushName(0);//make sure not empty
 	
   display();//draw and name
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glutPostRedisplay();//refresh
 	hits = glRenderMode(GL_RENDER);//get hits and change back to render

   for (int i = 0; i < hits; i++){
      printf("%d ",buff[i*4+ 3]);
   }
   printf("\n");
   
   //if (hits <1) return -1;
   //int i;
   int j = (int)hits;
   j--;
   while (j >= 0 && (buff[j*4 + 3] < 0 || buff[j*4 + 3] > 10000)){
      j--;
   }
   
   return buff[j*4 + 3];
}

void reshape(int w, int h) {
	GW = w;
	GH = h;
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
    gluPerspective(60, (float)w/h, 0.05, 100);

	glMatrixMode(GL_MODELVIEW);
	glViewport(0, 0, w, h);
	
	glutPostRedisplay();
}

void makeRain(){
   for (int i = -10; i < 60; i++){
      for (int j = -35; j < 17; j++){
         if ((rand() % 1000)< RAINDENS)
         rain.push_back(rainType(i+(rand()%100)/100,50 + (float)5*(rand()%100)/100,j+(rand()%100)/100,(float)(rand()%100 + 51)/100));
      }
   }
}

int checkMove(){
   for (unsigned int i = 0; i < objs.size(); i++){
    if (objs[i].hitTest(cam.px,cam.py-.5,cam.pz)){
       return 1;
    }
   }
   return 0;
}

void moveMe(float dx,float dz){
	cam.px += cam.x * dz - cam.z * dx;
   if (checkMove()){
      cam.px -= cam.x * dz - cam.z * dx;
   }
	cam.pz += cam.z * dz + cam.x * dx;
   if (checkMove()){
	   cam.pz -= cam.z * dz + cam.x * dx;
   }
}

void keyboardUp(unsigned char key, int x, int y ){
   switch( key ) {
    case 'w' : case 'W' : case 's' : case 'S' :
         dirz = 0;
         break;
      case 'a' : case 'A' : case 'd' : case 'D' :
         dirx = 0;
         break;
   }
}

void keyboard(unsigned char key, int x, int y ){
   switch( key ) {
      case 'q': case 'Q' :
         exit( EXIT_SUCCESS );
         break;
      case 'r': case 'R' :
         cam.px = path[lindex-1].x;
         cam.py = path[lindex-1].y;
         cam.pz = path[lindex-1].z;
         break;
       case 'p': case 'P' :
         mypause = !mypause;
         if (mypause == false) glutTimerFunc(TIMERINT, update, TIMERINT);
         break;
       case 'v': case 'V' :
         wireframe = !wireframe;
         break;
       case 't': case 'T' :
         makeRain();
         break;
      case 'y': case 'Y' :
         rainOn = !rainOn;
         break;
       case ' ':
         grav = -MAXJUMP;
         break;
      case 'w' : case 'W' :
         dirz = -1;//moveMe(0,-.2);
         break;
      case 's' : case 'S' :
         dirz = 1;//moveMe(0,.2);
         break;
      case 'a' : case 'A' :
         dirx = 1;//moveMe(.2,0);
         break;
      case 'd' : case 'D' :
         dirx = -1;//moveMe(-.2,0);
         break;
      case 'e' : case 'E' :
         lightdif = !lightdif;
         if (lightdif) {glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.01f);}
         else {glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.0f);}
         break;
   }
   glutPostRedisplay();
}

float p2wx(int xp) {
   return 2 * ((float)(2*GW*xp) / (GH*(GW-1)) - ((float)GW / GH));
}

float p2wy(int yp) {
   return 2 * (((float)(2*yp) / (GH-1)) - 1);
}

int g2py(int yp) {
   return (GH-1) - yp;
}

void updateCamera(){
	float r;
	r = cam.mag();
    cam.x = r*cos(phi*DEG2RAD)*cos(theta*DEG2RAD);
    cam.y = r*sin(phi*DEG2RAD);
    cam.z = r*cos(phi*DEG2RAD)*cos((90 - theta)*DEG2RAD);
}

void mouse(int button, int state, int x, int y) {
  trann = false;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) { /* if the left button is clicked */
       trann = true;
       lastx = p2wx(x);
       lasty = p2wy(g2py(y));
    }
  } else {
     if (state == GLUT_DOWN) { 
      int id = selector(x,y);
      //printf("%d",id);
         if (id >= 0){ 
            int od = id/6;
            float dx = objs[od].x;
            float dy = objs[od].y;
            float dz = objs[od].z;
            switch(id % 6){
             case 0:
                dz -= 1;
               break;
             case 1:
                dz += 1;
               break;
             case 2:
                dy += 1;
               break;
             case 3:
                dy -= 1;
               break;
             case 4:
                dx -= 1;
               break;
             case 5:
                dx += 1;
               break;
            }
            objs.push_back(newCube(dx,dy,dz,1,1,1));
         }
     }
  }
}

void mouseMove(int x, int y) {
  //update(0);
  //if (trann) {
         theta += 10*(p2wx(x) - lastx);
         phi -= 10*(p2wy(g2py(y)) - lasty);
		 if (phi > 70 || phi < -70){
			phi += 10*(p2wy(g2py(y)) - lasty);
		 }
         updateCamera();
         lastx = p2wx(x);
         lasty = p2wy(g2py(y));
         
         //reshape(GW,GH);
         //glutPostRedisplay();
         
   //}
   
}

bool rainCheck(int i){
   for (unsigned int j = 0; j < objs.size(); j++){
          if (objs[j].hitTest(rain[i].x,rain[i].y,rain[i].z)){
             rain[i].hit++;
             if (rain[i].hit > 20){
                rain.erase(rain.begin()+ i);
                return true;
             }
             return false;
          }
   }
   rain[i].y -= RSPEED;
   if (rain[i].y < -10 || rain[i].hit > 0){
      rain.erase(rain.begin()+ i);
      return true;
   }
   return false;
}

void makeItRain(){
   if (deg % 20 == 0 && rainOn) makeRain();
   for (unsigned int i = 0; i < rain.size(); i++){
      if (deg % 2 == 0){
         if (rainCheck(i)) i--;
      } else {
         if (rain[i].hit == 0) rain[i].y -= RSPEED;
      }
   }
}

void updateLight(){
   //if closer than .. move to text point
    light_spec[0] = .6*sin(DEG2RAD*deg+.5) + .4;
    light_spec[1] = .6*sin(DEG2RAD*deg*1.1+.5)+ .4;
    light_spec[2] = .6*sin(DEG2RAD*deg*1.2+.5)+ .4;
    light_amb[0] = .3*sin(DEG2RAD*deg)+ .7;
    light_amb[1] = .3*sin(DEG2RAD*deg*1.1)+ .7;
    light_amb[2] = .3*sin(DEG2RAD*deg*1.5)+ .7;
    light_diff[0] = .3*sin(DEG2RAD*deg)+ .7;
    light_diff[1] = .3*sin(DEG2RAD*deg*1.1)+ .7;
    light_diff[2] = .3*sin(DEG2RAD*deg*1.5)+ .7;
    if (idleL){
      light_pos[1] += ldy;
      ldy += ltog*IDLEACC;
      if (ldy > IDLESPEED || ldy < -IDLESPEED) ltog *= -1;
      float dist = distance(light_pos[0],light_pos[1],light_pos[2],cam.px,cam.py,cam.pz);
      if (dist < 2 && dist > -2){
         lindex++;
         if (lindex > path.size()-1){
            lindex = 0;
         }
         ldx = path[lindex].x - light_pos[0];
         ldy = path[lindex].y - light_pos[1];
         ldz = path[lindex].z - light_pos[2];
         dist = sqrt(ldx*ldx + ldy*ldy + ldz*ldz);
         ldx /= LSPEED*dist;
         ldy /= LSPEED*dist;
         ldz /= LSPEED*dist;
         idleL = false;
      }      
    } else {
      light_pos[0] += ldx;
      light_pos[1] += ldy;
      light_pos[2] += ldz;
      float dist = distance(light_pos[0],light_pos[1],light_pos[2],path[lindex].x,path[lindex].y,path[lindex].z);
      if (dist < .5 && dist > -.5){//light close to point
         ldy = 0;
         ltog = -1;
         idleL = true;
      }
   }
}

void gravity(){
   cam.py-=grav;
   grav+=GRAVACC;
   if (grav > GRAVTERM) grav = GRAVTERM;
   if (cam.py < -10){
      cam.px = path[lindex-1].x;
      cam.py = path[lindex-1].y;
      cam.pz = path[lindex-1].z;
      return;   
   }
   for (unsigned int i = 0; i < objs.size(); i++){
    if (objs[i].moving){
          objs[i].x += objs[i].mx*ospeed;
          objs[i].z += objs[i].mz*ospeed;  
    }
    if (objs[i].hitTest(cam.px,cam.py-1,cam.pz)){
       while (objs[i].hitTest(cam.px,cam.py-1,cam.pz))
          cam.py+=.01;
       grav = .1;
       if (objs[i].moving){
          cam.px += objs[i].mx*ospeed;
          cam.pz += objs[i].mz*ospeed;  
       }
    }
  }
}

void update(int n){
   glutWarpPointer(GW / 2, GH / 2);
         lastx = p2wx(GW / 2);
         lasty = p2wy(g2py(GH / 2));

   deg += 1;//.5;
   if (deg >= 360) {deg = 0;}
   ospeed = offset;
   offset = sin(DEG2RAD*deg);
   ospeed = offset - ospeed;
   if (dirx || dirz) moveMe(MAXSPEED*dirx,MAXSPEED*dirz);
   gravity();
   makeItRain();
   updateLight();
   glutPostRedisplay();
   if (!mypause) glutTimerFunc(TIMERINT, update, TIMERINT);
}

int main( int argc, char** argv ) {
  //set up my window
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL);
  glutInitWindowSize(800, 600); 
  glutInitWindowPosition(100, 100);
  glutCreateWindow("Shadow Game");
  glClearColor(0.0, 0.0, 0.0, 1.0);
  
  glutMouseFunc( mouse );
  glutPassiveMotionFunc( mouseMove );
  glutDisplayFunc( display );
  glutReshapeFunc( reshape );
  glutKeyboardFunc(keyboard);
  glutKeyboardUpFunc(keyboardUp);
  
   glutTimerFunc(5, update, 5);
	glClearDepth(1.0f);
   grav = .01;
   //glutWarpPointer(GW / 2, GH / 2);
   glutWarpPointer(400,300);
   glutSetCursor(GLUT_CURSOR_NONE);
   //lastx = p2wx(GW / 2);
   //lasty = p2wy(g2py(GH / 2));

  //enable z-buffer
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  
  //glEnable(GL_FOG);
  //glFogi (GL_FOG_MODE, GL_EXP2);
  //glFogf (GL_FOG_DENSITY, .1);

  //glHint(GL_POLYGON,GL_NICEST);

  glEnable(GL_BLEND);
  glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

  glEnable(GL_TEXTURE_2D);
  LoadTexture((char*)"concrete.bmp",1);//concrete.crate.bmp", 1);
  LoadTexture((char*)"light.bmp", 2);
  //quad=gluNewQuadric();
  //gluQuadricNormals(quad, GLU_SMOOTH);
  //gluQuadricTexture(quad, GL_TRUE);
  constructScene();
  buildPath();
  srand ( 42 );

  stencil = true;
  //initialization
  cam = vec3d(0,0,-1,0,2,0);
  theta = 90;
  phi = 0;
  deg = 0;
  dirx = 0;
  dirz = 0;
  updateCamera();
  init_lighting();

  //light path setup
  ldx = 0;ldy = 0;ldz = 0;
  lindex = 0;
  idleL = false;
  //create light sphere
  lSphere = gluNewQuadric();
  gluQuadricDrawStyle(lSphere, GLU_FILL);
  //make opengl generate the texture coordinates for the sphere
  gluQuadricTexture(lSphere, GL_TRUE);
  
   // LoadTexture("crate.bmp", 1);
  //LoadTexture("cloud5.bmp", 2);

  glutPostRedisplay();
  glutMainLoop();
}
