/* Object functions
 * Author Mike Buerli F2010 */
#include "obj.h"


void obj::makeShadow(surf s, float lx, float ly, float lz)//makes shadow volumes
{
	float v[4][3];
   float a[4][3];
	for (unsigned int i = 0; i < 4; i++){
		//s.verts[i].x += x;
		//s.verts[i].y += y;
		//s.verts[i].z += z;//points in correct places
      a[i][0] = (s.verts[i].x + x);//distance between verts and light (light to vert vector)
		a[i][1] = (s.verts[i].y + y);
		a[i][2] = (s.verts[i].z + z);

		v[i][0] = (s.verts[i].x + x - lx);//distance between verts and light (light to vert vector)
		v[i][1] = (s.verts[i].y + y - ly);
		v[i][2] = (s.verts[i].z + z - lz);
		normalize(v[i]);
		v[i][0] *= M_INFINITY;//make very long
		v[i][1] *= M_INFINITY;
		v[i][2] *= M_INFINITY;
		v[i][0] += x;//set back to surface pos
		v[i][1] += y;
		v[i][2] += z;
	}

	/* back cap */
	glBegin(GL_QUADS);
		glVertex3fv(v[3]);
		glVertex3fv(v[2]);
		glVertex3fv(v[1]);
		glVertex3fv(v[0]);
	glEnd();

	/* front cap */
	glBegin(GL_QUADS);
		glVertex3fv(a[0]);
		glVertex3fv(a[1]);
		glVertex3fv(a[2]);
		glVertex3fv(a[3]);
	glEnd();

   //sides
	glBegin(GL_QUAD_STRIP);
   //glVertex3fv(a[0]);
	//glVertex3fv(v[0]);
	for(int i = 0; i <= 4; i++) {
		glVertex3fv(a[i%4]);
		glVertex3fv(v[i%4]);
	}
   //glVertex3f(s.verts[0].x,s.verts[0].y,s.verts[0].z);
	//glVertex3fv(v[0]);
	glEnd();

	//for (unsigned int i = 0; i < 4; i++){
	//	s.verts[i].x -= x;
	//	s.verts[i].y -= y;
	//	s.verts[i].z -= z;//points back in correct places
   //}
}

void obj::drawObj(int index) {
   glPushMatrix();
     glTranslatef(x, y, z);
     //materials(o.material);
     for (unsigned int i = 0; i < surface.size(); i++){
         glLoadName(6*index+i);
         //  //set the current texture to be the crate
         
         if (wireframe){glBegin(GL_LINE_LOOP);}
         else {glBindTexture(GL_TEXTURE_2D, 1);glBegin(GL_POLYGON);}
            glNormal3fv(surface[i].n);
               glTexCoord2f(0,0);
               glVertex3f(surface[i].verts[0].x,surface[i].verts[0].y,surface[i].verts[0].z);
               glTexCoord2f(1,0);
               glVertex3f(surface[i].verts[1].x,surface[i].verts[1].y,surface[i].verts[2].z);
               glTexCoord2f(1,1);
               glVertex3f(surface[i].verts[2].x,surface[i].verts[2].y,surface[i].verts[2].z);
               glTexCoord2f(0,1);
               glVertex3f(surface[i].verts[3].x,surface[i].verts[3].y,surface[i].verts[3].z);
         glEnd(); 
     }
   glPopMatrix();
}

void obj::drawShadow(float lx, float ly, float lz)
{	
      for (unsigned int i = 0; i < surface.size(); i++){
      glDisable(GL_LIGHTING);
         glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
		      glDepthMask(GL_FALSE);
		         glEnable(GL_CULL_FACE);
		            glEnable(GL_STENCIL_TEST);

		            glEnable(GL_POLYGON_OFFSET_FILL);//makes so shadow not on top of surface
		            glPolygonOffset(0.0f, 100.0f);

		               glCullFace(GL_FRONT);
		               glStencilFunc(GL_ALWAYS, 0x0, 0xff);
		               glStencilOp(GL_KEEP, GL_INCR, GL_KEEP);
                     //glStencilOp(GL_KEEP, GL_KEEP,GL_INCR);
		               makeShadow(surface[i], lx, ly, lz);//keep back facing shadow
                     //printf("%d %d %d \n", o.x,o.y,o.z);
		               glCullFace(GL_BACK);
		               glStencilFunc(GL_ALWAYS, 0x0, 0xff);
                     glStencilOp(GL_KEEP, GL_DECR, GL_KEEP);
		               //glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
		               makeShadow(surface[i], lx, ly, lz);//mask out front facing shadow

		            glDisable(GL_POLYGON_OFFSET_FILL);
		         glDisable(GL_CULL_FACE);
            glDepthMask(GL_TRUE);
		   glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
		   

		   glStencilFunc(GL_NOTEQUAL, 0x0, 0xff);//get back and front caps of shadow
		   glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
		   shadowPlane();//draw actuall shadow
		   glDisable(GL_STENCIL_TEST);
      glEnable(GL_LIGHTING);
      }
}

bool obj::hitTest(float ix, float iy, float iz){
   float dx = (ix-x);
   float dy = (iy-y);
   float dz = (iz-z);
   if (dx < -0.1 || dx > bx+0.1) return false;
   if (dy < -0.1 || dy > by+0.1) return false;
   if (dz < -0.1 || dz > bz+0.1) return false;
   if (dy <= fx*(dx) + fz*(dz) + fy) return true;
   return false;
}

void shadowPlane()
{
   glMatrixMode(GL_MODELVIEW);
	glPushMatrix();//modelview
	   glLoadIdentity();
	   glMatrixMode(GL_PROJECTION);
	   glPushMatrix();
	      glLoadIdentity();
	      glOrtho(0, 1, 0, 1, 0, 1);
	      glDisable(GL_DEPTH_TEST);

	      glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
	      glBegin(GL_QUADS);
		      glVertex2i(0, 0);
		      glVertex2i(0, 1);
		      glVertex2i(1, 1);
		      glVertex2i(1, 0);
	      glEnd();

	      glEnable(GL_DEPTH_TEST);
	   glPopMatrix();
	   glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

float dot_product(float v1[3], float v2[3])
{
	return (v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2]);
}

void normalize(float v[3])
{
	float f = 1.0f / sqrt(dot_product(v, v));
	v[0] *= f;
	v[1] *= f;
	v[2] *= f;
}

obj newSlope(float x,float y,float z,float x2,float y2,float z2,int dirx, int m){
   float tmp;
   if (x2 < x) {
      tmp = x;
      x= x2;
      x2 = tmp;
   }
   if (z2 < z) {
      tmp = z;
      z= z2;
      z2 = tmp;
   
   }
   obj o = obj(x,y,z);
   o.fx = 0;
   o.fy = 0;
   o.fz = 0;
   o.moving=false;
   o.mx = 0;
   o.mz = 0;
   surf s = surf();
   s.n[0] = 0;
   s.n[2] = 0;
   float w = x2 -x;
   float h = y2 - y;
   float l = z2- z;
   o.bx = w;
   o.by = h; 
   o.bz = l;
if (dirx){
   s.verts.push_back(pnt3d(0,(1 - m)*h,0));
   s.verts.push_back(pnt3d(0,(1 - m)*h,l));
   s.verts.push_back(pnt3d(w,(m)*h,l));
   s.verts.push_back(pnt3d(w,(m)*h,0));
   if (m){
      o.fx = h/w;
      s.n[0] = w;
   } else {
      o.fx = -h/w;
      o.fy = h;
      s.n[0] = -w;
   }
} else {
   s.verts.push_back(pnt3d(0,(1 - m)*h,0));
   s.verts.push_back(pnt3d(0,(m)*h,l));
   s.verts.push_back(pnt3d(w,(m)*h,l));
   s.verts.push_back(pnt3d(w,(1 - m)*h,0));
   o.fz = (m)?(h/l):(-h/l);
   o.fy = (m)?(0):(h);
   if (m){
      o.fz = h/l;
      s.n[2] = l;
   } else {
      s.n[2] = -l;
      o.fz = -h/l;
      o.fy = h;
   }
}
   s.n[1] = h;
   normalize(s.n);
   o.surface.push_back(s); 
   return o;
}

obj newCube(float x,float y,float z,float dx,float dy,float dz){
   obj o = obj(x,y,z);
   surf s = surf();//front
   s.n[0] = 0;
   s.n[1] = 0;
   s.n[2] = 1;
   s.verts.push_back(pnt3d(0,dy,dz));
   s.verts.push_back(pnt3d(0,0,dz));
   s.verts.push_back(pnt3d(dx,0,dz));
   s.verts.push_back(pnt3d(dx,dy,dz));
   o.surface.push_back(s);
   s = surf();//back
   s.n[0] = 0;
   s.n[1] = 0;
   s.n[2] = -1;
   s.verts.push_back(pnt3d(dx,dy,0));
   s.verts.push_back(pnt3d(dx,0,0));
   s.verts.push_back(pnt3d(0,0,0));
   s.verts.push_back(pnt3d(0,dy,0));
   o.surface.push_back(s);
   s = surf();//top
   s.n[0] = 0;
   s.n[1] = 1;
   s.n[2] = 0;
   s.verts.push_back(pnt3d(0,dy,0));
   s.verts.push_back(pnt3d(0,dy,dz));
   s.verts.push_back(pnt3d(dx,dy,dz));
   s.verts.push_back(pnt3d(dx,dy,0));
   o.surface.push_back(s);
   s = surf();//bottom
   s.n[0] = 0;
   s.n[1] = -1;
   s.n[2] = 0;
   s.verts.push_back(pnt3d(dx,0,0));
   s.verts.push_back(pnt3d(dx,0,dz));
   s.verts.push_back(pnt3d(0,0,dz));
   s.verts.push_back(pnt3d(0,0,0));
   o.surface.push_back(s);
   s = surf();//left
   s.n[0] = -1;
   s.n[1] = 0;
   s.n[2] = 0;
   s.verts.push_back(pnt3d(0,dy,dz));
   s.verts.push_back(pnt3d(0,dy,0));
   s.verts.push_back(pnt3d(0,0,0));
   s.verts.push_back(pnt3d(0,0,dz));
   o.surface.push_back(s);
   s = surf();//right
   s.n[0] = 1;
   s.n[1] = 0;
   s.n[2] = 0;
   s.verts.push_back(pnt3d(dx,0,dz));
   s.verts.push_back(pnt3d(dx,0,0));
   s.verts.push_back(pnt3d(dx,dy,0));
   s.verts.push_back(pnt3d(dx,dy,dz));
   o.surface.push_back(s);
   o.fx = 0;
   o.fy = dy; 
   o.fz = 0;
   o.bx = dx;
   o.by = dy; 
   o.bz = dz;
   o.moving=false;
   o.mx = 0;
   o.mz = 0;
   return o;
}
